/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hdiv.taglib.html;

import javax.servlet.jsp.JspException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.taglib.html.HiddenTag;
import org.apache.struts.util.RequestUtils;
import org.apache.struts.util.ResponseUtils;
import org.hdiv.dataComposer.IDataComposer;
import org.hdiv.util.HDIVUtil;

/**
 * Renders an HTML <b>&lt;input&gt;</b> element of type hidden, populated from the
 * specified value or the specified property of the bean associated with our current
 * form. This tag is only valid when nested inside a form tag body.
 * 
 * @author Aritz Rabadan
 * @author Gorka Vicente
 * @see org.apache.struts.taglib.html.HiddenTag
 */
public class HiddenTagHDIV extends HiddenTag {

	/**
	 * Universal version identifier. Deserialization uses this number to ensure that
	 * a loaded class corresponds exactly to a serialized object.
	 */
	private static final long serialVersionUID = 3459988508044864957L;

	/**
	 * Commons logging instance
	 */
	private static final Log log = LogFactory.getLog(HiddenTagHDIV.class);

	/**
	 * Generated encoded value by HDIV composer
	 */
	private String encodedValue = null;

	/**
	 * Generate the required input tag, followed by the optional rendered text.
	 * Support for <code>write</code> property since Struts 1.1.
	 * 
	 * @exception JspException if a JSP exception has occurred
	 * @see org.hdiv.composer.IDataComposer#compose(String, String, boolean)
	 */
	public int doStartTag() throws JspException {

		try { 
			String hiddenValue = value;			
			Object lookupValue = null;
			
			if (value == null) {				
				// locate and return the specified property of the specified bean
				 lookupValue = RequestUtils.lookup(pageContext, name, property, null);

				if (lookupValue != null) {
					hiddenValue = lookupValue.toString();
				}
			}
								
			IDataComposer dataComposer = HDIVUtil.getDataComposer();						
			this.encodedValue = dataComposer.compose(prepareName(), hiddenValue, false);						 					
			
			// Render the result to the output writer
			ResponseUtils.write(this.pageContext, this.renderInputElement());
			
			// Is rendering the value separately requested?
			if (!write) {
				return (EVAL_BODY_TAG);
			}

			// Calculate the value to be rendered separately
			// * @since Struts 1.1
			String results = null;
			if (value != null) {
				results = ResponseUtils.filter(value);
			} else {
				if (lookupValue == null) {
					results = "";
				} else {
					results = ResponseUtils.filter(lookupValue.toString());
				}
			}

			// Render the result to the output writer
			ResponseUtils.write(pageContext, results);
			return (EVAL_BODY_TAG);
		
		} catch (JspException e) {
			throw e;
		}
	}

	/**
	 * Renders a fully formed &lt;input&gt; element.
	 * 
	 * @throws JspException
	 */
	protected String renderInputElement() throws JspException {
		
        // Create an appropriate "input" element based on our parameters
		StringBuffer results = new StringBuffer("<input");
        
        prepareAttribute(results, "type", this.type);
        prepareAttribute(results, "name", prepareName());
		prepareAttribute(results, "accesskey", getAccesskey());
		prepareAttribute(results, "accept", getAccept());
		prepareAttribute(results, "maxlength", getMaxlength());
		prepareAttribute(results, "size", getCols());
		prepareAttribute(results, "tabindex", getTabindex());
		prepareValue(results);
		results.append(this.prepareEventHandlers());
		results.append(this.prepareStyles());
		prepareOtherAttributes(results);
		results.append(this.getElementClose());
				
		return results.toString();
	}

    /**
     * Prepare the name element
     * @return The element name.
     */
    protected String prepareName() throws JspException {

        if (property == null) {
            return null;
        }

        // * @since Struts 1.1
        if(indexed) {
            StringBuffer results = new StringBuffer();
            prepareIndex(results, name);
            results.append(property);
            return results.toString();
        }

        return property;
    }	
	
	/**
	 * Render the value element.
	 * 
	 * @param results The StringBuffer that output will be appended to.
	 */
	protected void prepareValue(StringBuffer results) throws JspException {

		results.append(" value=\"");
		
		if (this.encodedValue != null) {
			results.append(this.formatValue(this.encodedValue));
			
		} else if (redisplay || !"password".equals(type)) {
			Object lookupValue = RequestUtils.lookup(pageContext, name, property, null);
			results.append(this.formatValue(lookupValue));
		}
		results.append("\"");
	}

    /**
	 * Return the given value as a formatted <code>String</code>. This
	 * implementation escapes potentially harmful HTML characters.
	 * 
	 * @param unformat The value to be formatted. <code>null</code> values will be
	 *            returned as the empty String "".
	 * @throws JspException if a JSP exception has occurred
	 */
    protected String formatValue(Object unformat) throws JspException {
    	
        if (unformat == null) {
            return "";
        }
        return ResponseUtils.filter(unformat.toString());
    }	
	
	/**
	 * Prepares an attribute if the value is not null, appending it to the the given
	 * StringBuffer.
	 * 
	 * @param handlers The StringBuffer that output will be appended to.
	 */
	protected void prepareAttribute(StringBuffer handlers, String name, Object value) {

		if (value != null) {
			handlers.append(" ");
			handlers.append(name);
			handlers.append("=\"");
			handlers.append(value);
			handlers.append("\"");
		}
	}

	/**
	 * 'Hook' to enable tags to be extended and additional attributes added.
	 * @param handlers The StringBuffer that output will be appended to.
	 */
	protected void prepareOtherAttributes(StringBuffer handlers) {

	}

}
